// Copyright - Simovits Consulting 2009
// Author: James Dickson.

#include "common.h"

int parseArguments(int argc, char **argv, TESTINFO *testInfo, char *strPemDump, BOOL *chosenCryptos, BOOL *chosenProtocols)
{
	if(argc == 2 && (strcmp(argv[1], "--printavailable") != 0) && (strcmp(argv[1], "--help") != 0))
	{
		// If nothing other than the IP-address is specified. 
		testInfo->strHostname = argv[1];
	}
	else
	{

		// Parse the arguments.
		for(int i=1; i < argc; i++)
		{
			char *str = argv[i];

			// Check for no-arguments
			if(strcmp(str, "--testsend") == 0)
			{
				testInfo->bTestSend = TRUE;
			}
			else if(strcmp(str, "--nocolor") == 0)
			{
				testInfo->bUseColors = FALSE;
			}
			else if(strcmp(str, "--ciscospeshul") == 0)
			{
				testInfo->bUseCiscoSpeshul = TRUE;
			}
			else if(strcmp(str, "--disableworkarounds") == 0)
			{
				testInfo->bUseWorkarounds = FALSE;
			}
			else if(strcmp(str, "--disablerenegotiation") == 0 || (strcmp(str, "--norn") == 0))
			{
				testInfo->bUseWorkarounds = FALSE;
			}
			else if(strcmp(str, "--strictpcigrade") == 0)
			{
				testInfo->bUseNessusCipherTest = FALSE;
			}
			else if(strcmp(str, "--testsip") == 0)
			{
				testInfo->testSIP = TRUE;
			}
			else if(strcmp(str, "-pr") == 0)
			{
				testInfo->bPrintCommunication = TRUE;
			}
			else if(strcmp(str, "--printcom") == 0)
			{
				testInfo->bPrintCommunication = TRUE;
			}
			else if(strcmp(str, "--servermode") == 0)
			{
				testInfo->bServerMode = TRUE;
			}		
			else if(strcmp(str, "--help") == 0)
			{
				printUsage();
				return 0;
			}	
			else if(strcmp(str, "--includeunsupported") == 0)
			{
				testInfo->bOnlyAvailableCiphers = FALSE;
			}
			else if(strcmp(str, "--printavailable") == 0)
			{
				testInfo->bPrintAvailableCiphers = TRUE;				
			}
			else if(strcmp(str, "--printcert") == 0)
			{
				testInfo->bPrintCert = TRUE;
				
			}
			else if(strcmp(str, "--smtptls") == 0)
			{
				testInfo->bStartTLS = TRUE;				
			}
			else if(strcmp(str, "--popssl") == 0)
			{
				testInfo->bStartSTLS = TRUE;				
			}
			else if(strcmp(str, "--explicitftps") == 0)
			{
				testInfo->bStartFTPS = TRUE;				
			}
			else if(strcmp(str, "--verbose") == 0)
			{
				testInfo->bVerbose = TRUE;			
			}
			else if(strcmp(str, "--nopct") == 0)
			{
				testInfo->bTestPCT = FALSE;		
			}

			if((i-1) < argc)
			{
				if(strcmp(str, "-t") == 0)
				{
					i++;
					testInfo->strHostname = argv[i];
				}
				else if(strcmp(str, "-h") == 0)
				{
					i++;
					testInfo->strGetHostname = argv[i];
				}
				else if(strcmp(str, "--page") == 0)
				{
					i++;
					testInfo->strHTTPPage = argv[i];
				}
				else if(strcmp(str, "--onlygrade") == 0)
				{
					i++;

					if(strcmpi(argv[i], "high") == 0)
					{
						testInfo->onlyGrade = 3;
					}
					else
					{
						testInfo->onlyGrade = atoi(argv[i]);
					}
				}
				else if(strcmp(str, "--certout") == 0)
				{
					i++;
					strcpy(strPemDump, argv[i]);
				}
				else if(strcmp(str, "--UDP") == 0)
				{
					// 2010-03-24 - james.
					testInfo->bUDP = TRUE;
				}
				else if(strcmp(str, "--servercipher") == 0)
				{
					i++;
					testInfo->strServerCipher = argv[i];
				}
				else if(strcmp(str, "--serverprotocol") == 0)
				{
					i++;
					
					if(strcmpi(argv[i], "SSL2") == 0)
					{
						testInfo->serverProtocol = SSL2;
					}
					else if(strcmpi(argv[i], "SSL3") == 0)
					{
						testInfo->serverProtocol = SSL3;
					}
					else if(strcmpi(argv[i], "TLS1") == 0)
					{
						testInfo->serverProtocol = TLS1;
					}
				}
				else if(strcmp(str, "--protocol") == 0)
				{
					i++;
					testInfo->bTestOnlySpecificProtocols = TRUE;

					if(strcmpi("SSL2", argv[i]) == 0)
					{
						chosenProtocols[0] = TRUE;
					}
					else if(strcmpi("SSL3", argv[i]) == 0)
					{
						chosenProtocols[1] = TRUE;
					}
					else if(strcmpi("TLS1", argv[i]) == 0)
					{
						chosenProtocols[2] = TRUE;
					}
					else
					{
						printf("Protocol not supported: %s\n", argv[i]);
						return -1;
					}
					
				}
				else if(strcmp(str, "-c") == 0)
				{
					i++;
					testInfo->strCACertPath = argv[i];
				}
				else if(strcmp(str, "--servercert") == 0)
				{
					i++;
					testInfo->strServerCertPath = argv[i];
				}
				else if(strcmp(str, "--servercertkey") == 0)
				{
					i++;
					testInfo->strServerCertKeyfile = argv[i];
				}
				else if(strcmp(str, "--servercertpass") == 0)
				{
					i++;
					testInfo->strServerCertPassword = argv[i];
				}
				else if(strcmp(str, "--sipdomain") == 0)
				{
					i++;
					testInfo->strSIPDomain = argv[i];
				}
				else if(strcmp(str, "-r") == 0)
				{
					i++;
					testInfo->strRequest = argv[i];
				}
				else if(strcmp(str, "-p") == 0)
				{
					i++;

					if(isInt(argv[i], (int) strlen(argv[i])))
					{
						testInfo->port = atoi(argv[i]);
					}
					else
					{	
						printf("ERROR: bad format of port number %s\n\n", argv[i]);
						printUsage();

						return -1;
					}
				}
				else if(strcmp(str, "--interval") == 0)
				{
					i++;

					if(isInt(argv[i], (int) strlen(argv[i])))
					{
						testInfo->msInterval = atoi(argv[i]);
					}
					else
					{	
						printf("ERROR: bad format of interval %s\n\n", argv[i]);
						printUsage();

						return -1;
					}
				}
				else if(strcmp(str, "--readtimeout") == 0)
				{
					i++;

					if(isInt(argv[i], (int) strlen(argv[i])))
					{
						// Assume milliseconds are given.
						testInfo->secondTimeout = atoi(argv[i]);
					}
					else
					{	
						printf("ERROR: bad format of read timeout %s\n\n", argv[i]);
						printUsage();

						return -1;
					}
				}
			}

		}
	}

	// Check the CA-cert path.
	if(testInfo->strCACertPath == NULL)
	{
		testInfo->strCACertPath = new char[256];
		getcwd(testInfo->strCACertPath, 256);
		strcat(testInfo->strCACertPath, "\\std-ca-bundle.crt");
	}

	// Check server certificate path
	if(testInfo->strServerCertPath == NULL)
	{
		testInfo->strServerCertPath = new char[256];
		getcwd(testInfo->strServerCertPath, 256);
		strcat(testInfo->strServerCertPath, "\\server.pem");
	}

	// Check server certificate path
	if(testInfo->strServerCertKeyfile == NULL)
	{
		testInfo->strServerCertKeyfile = new char[256];
		getcwd(testInfo->strServerCertKeyfile, 256);
		strcat(testInfo->strServerCertKeyfile, "\\server.pem");
	}

	// Get the correct IP-adress if we have a hostname.
	if(testInfo->strHostname != NULL && testInfo->strIPAdress == NULL)
	{		
		hostent *record = gethostbyname(testInfo->strHostname);	
		
		if(record != NULL && record->h_addr != NULL)
		{
			struct in_addr addr;
			addr.s_addr = *(u_long *) record->h_addr_list[0];
			testInfo->strIPAdress = inet_ntoa(addr);
		}
	}


	
	// This is used for HTTP-GET requests.
	if(testInfo->strGetHostname == NULL)
	{
		testInfo->strGetHostname = testInfo->strHostname;
	}

	return 2;
}