/**
 * SSLPressure 
 * Author: James Dickson.
 * License: GPL v3
 */
#include <stdio.h>
#include <winsock2.h>
#include <time.h>
#include "ssl2functions.h"
#include "pctfunctions.h"
#include "wsocket.h"



/**
 * Send TLS-ClientHello to server
 */
int pctSendClientHelloMessage(PCT_HELLO *msg, SOCKET lhSocket)
{
	int len = 80 + msg->CH_CIPHER_SPECS_LENGTH + msg->CH_HASH_SPECS_LENGTH + msg->CH_CERT_SPECS_LENGTH + msg->CH_EXCH_SPECS_LENGTH + msg->CH_KEY_ARG_LENGTH;
	int bufLen = len + 2;
	unsigned char *buf = (unsigned char *) malloc(bufLen); 
	int pos = 0;

	msg->length = len;

	buf[pos++] = 0x80;
	buf[pos++] = (BYTE) LOBYTE(msg->length);
	buf[pos++] = (BYTE) msg->CH_MSG_CLIENT_HELLO;
	buf[pos++] = (BYTE) HIBYTE(msg->CH_CLIENT_VERSION);
	buf[pos++] = (BYTE) LOBYTE(msg->CH_CLIENT_VERSION);
	buf[pos++] = (BYTE) msg->CH_PAD;

	for(int i=0; i < 64; i++)
	{
		buf[pos++] = 0;
	}

	//pos+=64;	// skip session id and data.


	buf[pos++] = (BYTE) HIBYTE(msg->CH_OFFSET);
	buf[pos++] = (BYTE) LOBYTE(msg->CH_OFFSET);
	buf[pos++] = (BYTE) HIBYTE(msg->CH_CIPHER_SPECS_LENGTH);
	buf[pos++] = (BYTE) LOBYTE(msg->CH_CIPHER_SPECS_LENGTH);
	buf[pos++] = (BYTE) HIBYTE(msg->CH_HASH_SPECS_LENGTH);
	buf[pos++] = (BYTE) LOBYTE(msg->CH_HASH_SPECS_LENGTH);
	buf[pos++] = (BYTE) HIBYTE(msg->CH_CERT_SPECS_LENGTH);
	buf[pos++] = (BYTE) LOBYTE(msg->CH_CERT_SPECS_LENGTH);
	buf[pos++] = (BYTE) HIBYTE(msg->CH_EXCH_SPECS_LENGTH);
	buf[pos++] = (BYTE) LOBYTE(msg->CH_EXCH_SPECS_LENGTH);
	buf[pos++] = (BYTE) HIBYTE(msg->CH_KEY_ARG_LENGTH);
	buf[pos++] = (BYTE) LOBYTE(msg->CH_KEY_ARG_LENGTH);

	for(int i=0; i < (msg->CH_CIPHER_SPECS_LENGTH/4); i++)
	{
		buf[pos++] = (BYTE) HIBYTE(msg->specs[i].cipher);
		buf[pos++] = (BYTE) LOBYTE(msg->specs[i].cipher);
		buf[pos++] = (BYTE) msg->specs[i].encryptionLength;
		buf[pos++] = (BYTE) msg->specs[i].macLength;
	}

	for(int i=0; i < (msg->CH_HASH_SPECS_LENGTH/2); i++)
	{
		buf[pos++] = (BYTE) HIBYTE(msg->CH_HASH_SPECS_DATA[i]);
		buf[pos++] = (BYTE) LOBYTE(msg->CH_HASH_SPECS_DATA[i]);
	}

	for(int i=0; i < (msg->CH_CERT_SPECS_LENGTH/2); i++)
	{
		buf[pos++] = (BYTE) HIBYTE(msg->CH_CERT_SPECS_DATA[i]);
		buf[pos++] = (BYTE) LOBYTE(msg->CH_CERT_SPECS_DATA[i]);
	}

	for(int i=0; i < (msg->CH_EXCH_SPECS_LENGTH/2); i++)
	{
		buf[pos++] = (BYTE) HIBYTE(msg->CH_EXCH_SPECS_DATA[i]);
		buf[pos++] = (BYTE) LOBYTE(msg->CH_EXCH_SPECS_DATA[i]);
	}

	// Send hello to server...
	int retval = send(lhSocket,(char *) buf, bufLen, 0);

	// Cleanup.
	free(buf);

	return retval;
}

/**
 * Test PCT.
 */
BOOL pctTestConnection(PCT_CIPHER *ciphers, WORD cipherLen, SOCKADDR_IN lSockAddr, int secondTimeout)
{
	SOCKET lhSocket = 0;
	BOOL bRet = FALSE;

	// Connect to server...
	int bConnected = getConnection(&lhSocket, &lSockAddr, secondTimeout);
	if(! bConnected)
	{
		return FALSE;
	}

	PCT_HELLO pct;
	BOOL bCiphersCreated = FALSE;

	if(ciphers == NULL)
	{
		cipherLen = 2;
		ciphers = (PCT_CIPHER *) malloc(sizeof(PCT_CIPHER) * cipherLen);

		ciphers[0].cipher = 0x0004;
		ciphers[0].encryptionLength = 0x80;
		ciphers[0].macLength = 0x40;

		ciphers[1].cipher = 0x0004;
		ciphers[1].encryptionLength = 0x28;
		ciphers[1].macLength = 0x40;

		bCiphersCreated = TRUE;
	}

	pct.CH_MSG_CLIENT_HELLO = 0x01;
	pct.CH_CLIENT_VERSION = SSL_VERSION_PCT1;
	pct.CH_PAD = 0x00;
	//unsigned char CH_SESSION_ID_DATA[32];
	//unsigned char CH_CHALLENGE_DATA[32];
	pct.CH_OFFSET = 0x000a;
	pct.CH_CIPHER_SPECS_LENGTH = cipherLen * 4;
	pct.CH_HASH_SPECS_LENGTH = 0x0004;
	pct.CH_CERT_SPECS_LENGTH = 0x0004;
	pct.CH_EXCH_SPECS_LENGTH = 0x0002;
	pct.CH_KEY_ARG_LENGTH = 0x00;

	pct.specs = ciphers;

	pct.CH_HASH_SPECS_DATA = (WORD *) malloc(pct.CH_HASH_SPECS_LENGTH + 1);
	pct.CH_CERT_SPECS_DATA = (WORD *) malloc(pct.CH_CERT_SPECS_LENGTH + 1);
	pct.CH_EXCH_SPECS_DATA = (WORD *) malloc(pct.CH_EXCH_SPECS_LENGTH + 1);

	pct.CH_HASH_SPECS_DATA[0] = 0x0001;
	pct.CH_HASH_SPECS_DATA[1] = 0x0003;

	pct.CH_CERT_SPECS_DATA[0] = 0x0003;
	pct.CH_CERT_SPECS_DATA[1] = 0x0001;

	pct.CH_EXCH_SPECS_DATA[0] = 0x0001;

	int retval = pctSendClientHelloMessage(&pct, lhSocket);

	free(pct.CH_HASH_SPECS_DATA);
	free(pct.CH_CERT_SPECS_DATA);
	free(pct.CH_EXCH_SPECS_DATA);
	

	// Receive server-hello
	const int PCTBUFSIZE = 8192;
	unsigned char sockBuf[PCTBUFSIZE];
	int rsize = recv(lhSocket, (char *) sockBuf, PCTBUFSIZE, 0);

	// Destroy connection
	closesocket(lhSocket);

	if((rsize > 8) && (sockBuf[2] = PCT1_HANDSHAKE_TYPE_SERVER_HELLO))
	{
		bRet= TRUE;
	}

	if(bCiphersCreated)
	{
		free(ciphers);
	}
	

	return bRet;
}