/**
 * SSLPressure 
 * Author: James Dickson.
 * License: GPL v3
 */
#include <stdio.h>
#include <winsock2.h>
#include <time.h>
#include "tlsfunctions.h"
#include "wsocket.h"

BOOL tlsGetHeader(BYTE *sockBuf, TLS_RECORD *tlsRecord)
{
	tlsRecord->contentType = sockBuf[0];
	tlsRecord->version = (sockBuf[1] << 8) | sockBuf[2];
	tlsRecord->helloLength = (sockBuf[3] << 8) | sockBuf[4];

	return TRUE;
}

BOOL tlsTestConnection(
						   //WORD tls_version_not_used_here, 
						   WORD *cipher, 
						   TLS_EXTENSION *extensions, 
						   WORD extensionLength, 
						   DWORD cipherCount,
						   SOCKADDR_IN lSockAddr,
						   int secondTimeout,
						   WORD currentSSLVersion, 
						   WORD *foundCipher, 
						   DWORD dwCipher_not_used_here,
						   DWORD *dwFoundCipher_not_used_here)
{
	SOCKET lhSocket = 0;

	// Connect to server...
	int bConnected = getConnection(&lhSocket, &lSockAddr, secondTimeout);
	if(! bConnected)
	{
		return FALSE;
	}

	// Send client-hello
	TLS_CLIENT_HELLO_START msg1;
	tlsCreateHelloMessage(&msg1, currentSSLVersion, cipher, NULL, 0, cipherCount);							
	tlsSendClientHelloMessage(&msg1, lhSocket);

	free(msg1.cipherSuite);

	// Receive server-hello
	char sockBuf[1024];
	int rsize = recv(lhSocket, sockBuf, 1024, 0);

	// Destroy connection
	closesocket(lhSocket);

	// Found at least one valid, so we go through the whole block
	if(rsize > 44 && sockBuf[0] == 0x16)
	{
		//TLS_RECORD tlsRecord;				
		//tlsGetHeader((BYTE*) sockBuf, &tlsRecord);

		int idLen = sockBuf[43];
		WORD cCiper = sockBuf[43 + idLen + 1] | sockBuf[43 + idLen + 2];

		*foundCipher = cCiper;

		return TRUE;
	}
	return FALSE;
}

/**
 * Create the TLS-hello message 
 */
void tlsCreateHelloMessage(TLS_CLIENT_HELLO_START *msg, WORD tls_version, 
						   WORD *cipher, TLS_EXTENSION *extensions, WORD extensionLength, DWORD cipherCount)
{
	__time32_t t = _time32(NULL);
	DWORD helloLength = TLS_STARTBYTES + (cipherCount *2);	// The full clientHello minus the length of the extensions and all but one ciphers.

	helloLength += extensionLength;
	if(extensionLength > 0)
	{
		helloLength += 2; // 2 extra bytes for the extension-length-bytes.
	}

	// Check extensions.
	if(extensionLength > 0)
	{
		msg->extensions = extensions;
	}
	else
	{
		msg->extensions = NULL;
	}

	// RECORD
	msg->tlsRecord.contentType = TLS_CONTENT_TYPE_HANDSHAKE;	
	msg->tlsRecord.version = tls_version;
	msg->tlsRecord.helloLength = (WORD) helloLength;

	// ClientHello
	DWORD dwTemp = (helloLength - 4);	// Subtract the 4 bytes of the record-header.
	msg->tlsClientHello.handshakeType = TLS_CLIENTHELLO_HANDSHAKE_TYPE;
	msg->tlsClientHello.length[0] = (unsigned char) (dwTemp >> 16);
	msg->tlsClientHello.length[1] = (unsigned char) (dwTemp >> 8);
	msg->tlsClientHello.length[2] = (unsigned char) (dwTemp);
	msg->tlsClientHello.version = tls_version;

	// Random
	msg->tlsRandom.strRandom[0] = 0xCC; // Just a marker... don't care about the rest
	//memcpy((void *) msg->tlsRandom.strTime, (void *) &t, 4);

	// Session id length
	msg->sessionIDLength = 0x00;

	// Cipher suites length
	msg->cipherSuiteLength = (WORD) (cipherCount * 2);

	// Cipher suites
	msg->cipherSuite = (unsigned char *) malloc(msg->cipherSuiteLength + 1); //new unsigned char[msg->cipherSuiteLength + 1];	
	int count = 0;
	for(int i=0; i < msg->cipherSuiteLength; i+=2)
	{
		msg->cipherSuite[i] = HIBYTE(cipher[count]);
		msg->cipherSuite[i+1] = LOBYTE(cipher[count++]);
	}
	

    //Compression Methods
	msg->compressionsMethodsLength = 0x01;
	msg->compressionsMethods = 0x00;	

    //Extentions Length
	msg->extensionsLength = extensionLength;

}

/**
 * Send TLS-ClientHello to server
 */
int tlsSendClientHelloMessage(TLS_CLIENT_HELLO_START *msg, SOCKET lhSocket)
{
	int len = msg->tlsRecord.helloLength  + 5; // The length + the 5 bytes of the header.
	//if(msg->extensionsLength > 0) len+=2;

	unsigned char *buf = (unsigned char *) malloc(len); 
	int pos = 0;

	buf[pos++] = (BYTE) msg->tlsRecord.contentType;			// 0x16
	buf[pos++] = (BYTE) (msg->tlsRecord.version >> 8);		// 0x03
	buf[pos++] = (BYTE) msg->tlsRecord.version;				// 0x01
	buf[pos++] = (BYTE) (msg->tlsRecord.helloLength >> 8);	// 0x00 
	buf[pos++] = (BYTE) msg->tlsRecord.helloLength;			// 0x32


	buf[pos++] = msg->tlsClientHello.handshakeType;			// 0x01
	buf[pos++] = msg->tlsClientHello.length[0];				// 0x00
	buf[pos++] = msg->tlsClientHello.length[1];				// 0x00
	buf[pos++] = msg->tlsClientHello.length[2];				// 0x2e
	buf[pos++] = (BYTE) (msg->tlsClientHello.version >> 8);	// 0x03
	buf[pos++] = (BYTE) msg->tlsClientHello.version;		// 0x01

	// Time... not important for this app.
	buf[pos++] = 0x4d; // (BYTE) msg->tlsRandom.strTime[0];			
	buf[pos++] = 0x28; // (BYTE) msg->tlsRandom.strTime[1];
	buf[pos++] = 0x73; // (BYTE) msg->tlsRandom.strTime[2];
	buf[pos++] = 0x3d; // (BYTE) msg->tlsRandom.strTime[3];
	//pos += 28;

	for(int x=0; x < 28; x++)
	{
		buf[pos++] = 0xFF;
	}

	// Session id length
	buf[pos++] = msg->sessionIDLength;						// 0x00

	// Cipher suites length
	buf[pos++] = (BYTE) (msg->cipherSuiteLength >> 8);		// 0x00
	buf[pos++] = (BYTE) msg->cipherSuiteLength;				// 0x02

	// Cipher suites
	for(int i=0; i < msg->cipherSuiteLength; i++)
	{
		buf[pos++] = msg->cipherSuite[i];						// 0xc0
		//buf[pos++] = msg->cipherSuite[1];						// 0xca
	}

    //Compression Methods
	buf[pos++] = msg->compressionsMethodsLength;			// 0x01
	buf[pos++] = msg->compressionsMethods;					// 0x00

    //Extensions
	if(msg->extensionsLength > 0)
	{
		buf[pos++] = HIBYTE(msg->extensionsLength);			// 0x00
		buf[pos++] = LOBYTE(msg->extensionsLength);			// 0x00

		BOOL bDone = FALSE;
		DWORD curPos = 0;

		while(!bDone)
		{
			TLS_EXTENSION *tEx = (TLS_EXTENSION *) (msg->extensions + curPos);
			
			buf[pos++] = HIBYTE(tEx->type);
			buf[pos++] = LOBYTE(tEx->type);

			buf[pos++] = HIBYTE(tEx->length);
			buf[pos++] = LOBYTE(tEx->length);

			for(int x=0; x < tEx->length; x++)
			{
				buf[pos++] = tEx->data[x];
			}

			curPos += tEx->length + 4;

			if(curPos >= msg->extensionsLength)
			{
				bDone = TRUE;
			}
		}
	}

	//printf("len=%d, pos=%d\n", len, pos);

	int retval = send(lhSocket,(char *) buf, len, 0);

	free(buf);

	return retval;
}
