/**
 * SSLPressure 
 * Author: James Dickson.
 * License: GPL v3
 */
#ifndef TLS_FUNCTIONS_H
#define TLS_FUNCTIONS_H

#define TLS_CONTENT_TYPE_HANDSHAKE 0x16;
#define TLS_CLIENTHELLO_HANDSHAKE_TYPE 0x01;
#define TLS_STARTBYTES 43

#define SSL_VERSION_PCT1	0x8001
#define SSL_VERSION_2		0x0002
#define SSL_VERSION_3		0x0300
#define SSL_VERSION_TLS1	0x0301
#define SSL_VERSION_TLS11	0x0302
#define SSL_VERSION_TLS12	0x0303

#define EXTENSION_TYPE_RFC5746	0xFF01



// 5 bytes
typedef struct tagTLS_RECORD
{
	BYTE contentType;
	WORD version;
	WORD helloLength;
}TLS_RECORD;

// 6 Bytes
typedef struct tagTLS_CLIENTHELLO
{
	BYTE handshakeType;
	BYTE length[3];
	WORD version;
}TLS_CLIENTHELLO;

// 32 Bytes
typedef struct tagTLS_RANDOM
{
	unsigned char strTime[4]; 
    unsigned char strRandom[28];
}TLS_RANDOM;

// 4+ bytes
typedef struct tagTLS_EXTENSION
{
	WORD type;
	WORD length;
	unsigned char *data;

}TLS_EXTENSION;

// 
typedef struct tagTLS_CLIENT_HELLO_START
{
	TLS_RECORD tlsRecord;
	TLS_CLIENTHELLO tlsClientHello;
	TLS_RANDOM tlsRandom;
	BYTE sessionIDLength;
	WORD cipherSuiteLength;
	unsigned char *cipherSuite;		// 2 bytes at least
	BYTE compressionsMethodsLength;
	BYTE compressionsMethods;
	WORD extensionsLength;
	TLS_EXTENSION *extensions;

}TLS_CLIENT_HELLO_START;


// Define the function pointer
typedef BOOL (*CALLBACK_TestConnection) (
						   //WORD tls_version, 
						   WORD *cipher, 
						   TLS_EXTENSION *extensions, 
						   WORD extensionLength, 
						   DWORD cipherCount,
						   SOCKADDR_IN lSockAddr,
						   int secondTimeout,
						   WORD currentSSLVersion,
						   WORD *foundCipher, 
						   DWORD dwCipher,
						   DWORD *dwFoundCipher);

void tlsCreateHelloMessage(TLS_CLIENT_HELLO_START *msg, WORD tls_version, 
						   WORD *cipher, TLS_EXTENSION *extensions, WORD extensionLength, DWORD cipherCount);
int tlsSendClientHelloMessage(TLS_CLIENT_HELLO_START *msg, SOCKET lhSocket);
BOOL tlsGetHeader(BYTE *sockBuf, TLS_RECORD *tlsRecord);

BOOL tlsTestConnection(
						   //WORD tls_version, 
						   WORD *cipher, 
						   TLS_EXTENSION *extensions, 
						   WORD extensionLength, 
						   DWORD cipherCount,
						   SOCKADDR_IN lSockAddr,
						   int secondTimeout,
						   WORD currentSSLVersion,
						   WORD *foundCipher, 
						   DWORD dwCipher,
						   DWORD *dwFoundCipher);


#endif
